<?php

namespace Tool\Indexmenow;

Use Tool\Indexmenow\Controller\AdminController;
Use Tool\Indexmenow\Controller\SendLinkController;
Use Tool\Indexmenow\Controller\ManualButtonController;

class IndexMeNowPlugin {


        public function __construct($file) {
            add_action('plugins_loaded',[$this,'indexmenow_load_textdomain']);
            register_activation_hook($file,[$this,'activation_plugin']);
            register_deactivation_hook($file,[$this,'deactivation_plugin']);
            add_filter('plugin_action_links_indexmenow/indexmenow.php', [$this,'indexmenow_settings_links_extension_page']);
            add_action('init', [$this,'indexmenow_gutenberg_register_meta']);
            add_action( 'rest_api_init',[$this,'route_indexmenow_api']);
            add_action('admin_notices',[$this,'notice_activation_plugin']);
            add_action('enqueue_block_editor_assets',[$this,'indexmenow_module_gutenberg']);
            add_action( 'cron_schedules', [$this,'thirty_minutes_interval'] );
            add_action('indexmenow_auto_task_hook',[$this,'auto_index_publish']);

            if(is_admin()){

                $adminController = new AdminController();
                $sendLinkController = new SendLinkController();
                $ManualButtonController = new ManualButtonController();
            }

        }
        public function indexmenow_load_textdomain() {
            load_plugin_textdomain( 'indexmenow', false, dirname( plugin_basename(__FILE__)).'/languages') ;
        }
        //add schedules every thirty minutes
        function thirty_minutes_interval( $schedules ) { 
            $display = __('Every 30 Minutes','indexmenow');
            $schedules['thirtyminutes'] = array(
                'interval' => 1800,
                'display'  => $display, 
            );
            return $schedules;
        }
         //Activation plugin
         public function activation_plugin(){
             
            set_transient('index_me_now_activate_plugin',true);

            if(!wp_next_scheduled('indexmenow_auto_task_hook')){

                wp_schedule_event(time(),'thirtyminutes','indexmenow_auto_task_hook' );
                
            }
            
         }
        //Deactivation plugin
        public function deactivation_plugin(){

                wp_unschedule_event(time(),'thirtyminutes','indexmenow_auto_task_hook' );
                wp_clear_scheduled_hook( 'indexmenow_auto_task_hook' ); 
         }
         //Notice activation plugin
         public function notice_activation_plugin(){
    
            if(get_transient('index_me_now_activate_plugin') == true){

                self::render('notices',[
                    'message'=>__('IndexMeNow Plugin has been activated. Add your API key in','indexmenow').' <a href="'.menu_page_url('index-me-now',$echo = false).'">'.__('Configuration','indexmenow').'</a>'
                ]);
                
                update_option( 'data_indexmenow_auto', 'off' );
                update_option( 'data_indexmenow_api', '');
                update_option( 'statut_indexmenow_api', 'KO' );
                update_option( 'code_error_indexmenow_api','');
                update_option( 'credit_indexmenow_api',0);
                delete_transient( 'index_me_now_activate_plugin' );
            }
                

         }

        public function indexmenow_settings_links_extension_page($links)
        {
            $links[] = '<a href='.menu_page_url('index-me-now',$echo = false).'>' . __('Settings', 'indexmenow') . '</a>';

            return $links;
        }
         //Render template view
         public static function render(string $name, array $args=[] ){
             extract($args); 
             $file = INDEXMENOW_PLUGIN_DIR."views/$name.php";
             ob_start();
              include $file;
             echo ob_get_clean();
         }
        //check status account Indexmenow
        public static function checkAccountIndexmenow($key){
            
            $apikey = $key;
            $args = array(
                'headers' => array(
                    'Authorization' => 'Bearer '.$apikey,
                ),
                'method'    => 'GET',
                'timeout' => 120
            );
    
            $requete = wp_remote_request('https://tool.indexmenow.com/api/v1/user/credits',$args);

            $body = wp_remote_retrieve_body($requete);
            $decode_data = json_decode($body);
            $response = [
                'statut_api' => $decode_data->status,
                'error' => (isset($decode_data->error)) ? $decode_data->error : false,
                'credits'=> isset($decode_data->credits) ? $decode_data->credits : 0
            ];
           
            return $response;
    
        }
        //redirect to referer url
         public static function redirect() {
            if ( ! isset( $_POST['_wp_http_referer'] ) ) { 
                $_POST['_wp_http_referer'] = wp_login_url();
            }
        
            
            $url = sanitize_text_field(
                    wp_unslash( $_POST['_wp_http_referer'] ) 
            );
        
            wp_safe_redirect( urldecode( $url ) );
            exit;
         }
         
        //check if valide nonce
        public static function has_valid_nonce($name_field,$field,$action) {
        
            if ( !isset( $_POST[$name_field] ) ) { 
                return false;
            }
            return wp_verify_nonce($field, $action );

        }
        public function indexmenow_module_gutenberg(){

            $url = parse_url(get_site_url());
            $project_name =  $url['host'].' (plugin wordpress)';

            wp_enqueue_script(
                'indexmenow-sidebar',
                plugins_url( '../build/index.js', __FILE__ ),
                [ 
                  'wp-plugins',
                  'wp-edit-post',
                  'wp-i18n',
                  'wp-element', 
                  'wp-components',
                  'wp-data',
                 ]
                
            );
            wp_set_script_translations( 'indexmenow-sidebar', 'indexmenow', plugin_dir_path(__FILE__) . '/languages' );
            wp_localize_script('indexmenow-sidebar','config',array(
                'auto_index_activate' => get_option('data_indexmenow_auto'),
                'api_key_indexmenow' => get_option('data_indexmenow_api'),
                'statut_indexmenow_api' => get_option('statut_indexmenow_api'),
                'code_error_indexmenow_api' => get_option('code_error_indexmenow_api'),
                'project_name' =>  $project_name,
                'admin_url_page_indexmenow' => menu_page_url('index-me-now',$echo = false),

            ));
        }
        //Register Metaboxes
        public function indexmenow_gutenberg_register_meta_func( $postType, $fieldname, $type ) {
            register_post_meta( $postType, $fieldname, array(
                'show_in_rest' => true,
                'type' => $type,
                'single' => true,
                'sanitize_callback' => 'sanitize_text_field',
                'auth_callback' => function() {
                    return current_user_can('edit_posts');
                }
                
            ));
        }
        
        public function indexmenow_gutenberg_register_meta() {

            $this->indexmenow_gutenberg_register_meta_func('post', 'manual_indexmenow', 'boolean'); 
            $this->indexmenow_gutenberg_register_meta_func('page', 'manual_indexmenow', 'boolean'); 
            $this->indexmenow_gutenberg_register_meta_func('post', 'indexmenow_id_project','integer'); 
            $this->indexmenow_gutenberg_register_meta_func('page', 'indexmenow_id_project', 'integer');
    
        }

        public function route_indexmenow_api(){

            register_rest_route( 
              'indexmenow/v1','/api_indexmenow/(?P<id>\d+)', array(
              'methods' => 'POST',
              'callback' => [$this,'api_indexmenow'],
              'permission_callback' => '__return_true'
            ) );

        } 
        
        public function autosendLinkToApiGutenberg($post){
              
                $apikey = get_option('data_indexmenow_api');
                $url = parse_url(get_site_url());
                $name =  $url['host'].' (plugin wordpress)';
                    $project_name = array('project_name' =>  $name);
                    $args = array(
                        'headers' => array(
                            'Authorization' => 'Bearer '.$apikey,
                        ),
                        'body' => json_encode($project_name),
                        'method'    => 'POST',
                        'timeout' =>10
                        
                    );

                    $requete = wp_remote_request('https://tool.indexmenow.com/api/v1/project/exists',$args );
            
                    $decode_data = json_decode($requete["body"]);
                
                    $statut = $decode_data->status;
    
                    if($statut =='OK'){
                        
                        $id_project = $decode_data->project_id;
                    
                        $urls = array('urls' => array(get_permalink($post->ID)) );
                        
                        $args = array(
                        'headers' => array(
                            'Authorization' => 'Bearer '.$apikey,
                        ),
                        'body' => json_encode($urls),
                        'method'    => 'POST',
                        'timeout' =>10
                        
                        );
                    
                        $requete = wp_remote_request('https://tool.indexmenow.com/api/v1/project/'.$id_project.'/addurls',$args );
                        update_post_meta( $post->ID,'manual_indexmenow', true);
                        return  $id_project;
                        
                    }else{
    
                        $data = array(
                            'project_name' => $name,
                            'urls' => array(get_permalink($post->ID))
                        );
                
                        $args = array(
                            'headers' => array(
                                'Authorization' => 'Bearer '.$apikey,
                            ),
                            'body' => json_encode($data),
                            'method'    => 'POST',
            
                            
                        );
                        $requete = wp_remote_request('https://tool.indexmenow.com/api/v1/project/add',$args );
                        update_post_meta( $post->ID,'manual_indexmenow', true);
                        $requete_data = json_decode($requete["body"]);
                        return $requete_data->project_id;
                        
                    }
        }

        public function api_indexmenow($data){

            $post = get_post($data->get_param('id'));
        
            $apikey = get_option('data_indexmenow_api');
            $update_credits = $this->checkAccountIndexmenow($apikey);
            if($update_credits['credits'] > 0){
                $id_project = $this->autoSendLinkToApiGutenberg($post);
                update_option( 'credit_indexmenow_api',  $update_credits['credits'] );
                $result =[
                    'credits' => $update_credits['credits'],
                    'id_project' =>$id_project
                ];
                return  wp_send_json_success($result);

            }else{
                $result =[
                    'credits' => $update_credits['credits'],
                ];
                return wp_send_json_success($result);
            }
           
        }

        public function auto_index_publish(){

            if(get_option('data_indexmenow_auto') =='on' && get_option('data_indexmenow_api')){
                $posts = get_posts([
                    'post_type' => array('page','post'),
                    'post_status' => 'publish',
                    'numberposts' => -1
                    // 'order'    => 'ASC'
                ]);
                foreach($posts as $post ){
                    $post_date = get_the_date('Y-m-d',$post->ID);
                    $date_autopub = get_option('data_indexmenow_date_autopub');
               
                    if($date_autopub !=0 && $post_date >= $date_autopub ){

                        if(! get_post_meta($post->ID,'manual_indexmenow')){
                            $this->autosendLinkToApiGutenberg(get_post($post));
                        }

                    }
                    
                       
                }
            }

        }

}
